package cz.upol.zp4jv.lecture10;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

public class MovieDatabase implements AutoCloseable {
	
	private PreparedStatement getAllNames;
	private PreparedStatement getByNameStmt;
	private PreparedStatement insertStmt;
	private PreparedStatement updateStmt;
	private PreparedStatement deleteStmt;
	
	public MovieDatabase(Connection con) throws MovieDBException {
		try {
			getAllNames   = con.prepareStatement("SELECT name FROM movies");
			getByNameStmt = con.prepareStatement("SELECT * FROM movies WHERE (name = ?) AND (released_in = ?)");
			insertStmt    = con.prepareStatement("INSERT INTO movies (name, released_in, director, rating) VALUES (?, ?, ?, ?)");
			updateStmt    = con.prepareStatement("UPDATE movies SET rating = ? WHERE (name = ?) AND (released_in = ?)");
			deleteStmt    = con.prepareStatement("DELETE FROM movies WHERE (name = ?) AND (released_in = ?)");
		} catch (SQLException e) {
			throw new MovieDBException("Unable to initialize prepared statements.", e);
		}
	}
	
	public List<String> getMovieNames() throws MovieDBException {
		List<String> names = new ArrayList<String>();
		try (ResultSet results = getAllNames.executeQuery()) {
			while (results.next())
				names.add(results.getString(1));
		} catch (Exception e) {
			throw new MovieDBException("Unable to list all movies", e);
		}
		return names;
	}
	

	public Movie getByName(String name, int year) throws MovieDBException {
		Movie movie = null;
		try {
			getByNameStmt.setString(1, name);
			getByNameStmt.setInt(2, year);
			try (ResultSet results = getByNameStmt.executeQuery()) {
				if (results.next())
					movie = new Movie(results.getString("name"),
									  results.getInt("released_in"),
									  results.getString("director"),
									  results.getDouble("rating"));
			}
		} catch (SQLException e) {
			throw new MovieDBException("Unable to find a movie", e);
		}
		
		return movie;
	}
	
	public Movie create(String name, int releasedIn, String director, double rating) throws MovieDBException {
		try {
			insertStmt.setString(1, name);
			insertStmt.setInt(2, releasedIn);
			insertStmt.setString(3, director);
			insertStmt.setDouble(4, rating);
			insertStmt.executeUpdate();
			 			
			return new Movie(name, releasedIn, director, rating);
		} catch (SQLException e) {
			throw new MovieDBException("Unable to create new movie", e);
		}
	}
	
	public void remove(Movie movie) throws MovieDBException {
		try {
			deleteStmt.setString(1, movie.getName());
			deleteStmt.setInt(2, movie.getReleasedIn());
			deleteStmt.executeUpdate();
		} catch (SQLException e) {
			throw new MovieDBException("Unable to delete movie", e);
		}
	}
	
	/** ulozi zmeny provedene v objektu */
	public void commitChanges(Movie movie) throws MovieDBException {
		try {
			updateStmt.setDouble(1, movie.getRating());
			updateStmt.setString(2, movie.getName());
			updateStmt.setInt(3, movie.getReleasedIn());
			
			updateStmt.executeUpdate();
		} catch (SQLException e) {
			throw new MovieDBException("Unable to update movie", e);
		}
	}
	
	@Override
	public void close() {
		try {
			getAllNames.close();
			getByNameStmt.close();
			insertStmt.close();
			updateStmt.close();
			deleteStmt.close();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}


}
